@@ -11,6 +11,9 @@ from schematic.models.commands import model as model_cli  # submit manifest comm
 from schematic.schemas.commands import (
     schema as schema_cli,
 )  # schema conversion commands
+from schematic.visualization.commands import (
+    viz as viz_cli,
+)  # viz generation commands
 from schematic import init as init_cli  # schematic initialization commands
 
 logger = logging.getLogger()
@@ -34,6 +37,8 @@ main.add_command(init_cli)  # add init commands
 main.add_command(manifest_cli)  # add manifest commands
 main.add_command(model_cli)  # add model commands
 main.add_command(schema_cli)  # add schema commands
+main.add_command(viz_cli)  # add viz commands
+
 
 
 if __name__ == "__main__":
